{===============================================================================
  GetInfo (c)1997 by RicoSoft
  Riccardo 'Rico' Pareschi - rico@mbox.4net.it
 ===============================================================================
   An extract from my System Invesigations (see on Delphi Super Page) to study
   how to build a DLL in Delphi 3
   Please use, share & enjoy.
 ===============================================================================
}
unit GetInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Registry;


{- Structure to hold the results }
{$I SysInfo.inc}


function  GetAllSystemInfo: SystemInfoRecord; stdcall;

function  DllVersion: shortstring; stdcall;

{===============================================================================
                              IMPLEMENTATION
===============================================================================}
implementation

var
  OSVerInfo: TOSVersionInfo;


{===============================================================================
 small internal function to retrieve register informations
}
function GetRegStr(Key, St: string): string;
begin
    with TRegistry.Create do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey(Key, False) then result := ReadString(St);
    finally
      Free;
    end; {- try }
end; {- GetRegStr }


{===============================================================================
}
procedure GetComputerName;
var
   Computer : PChar;
   CSize    : DWORD;
begin
   CSize := MAX_COMPUTERNAME_LENGTH + 1;
   try
      GetMem( Computer, CSize );
      if Windows.GetComputerName( Computer, CSize ) then
         SysInfoRec.ComputerName := Computer;
   finally
      FreeMem( Computer );
   end;
end; {- GetComputerName }


{===============================================================================
}
procedure GetVolumeInfo;
var
   lpRootPathName           : PChar; // address of root directory of the file system
   lpVolumeNameBuffer       : PChar; // address of name of the volume
   nVolumeNameSize          : DWORD; // length of lpVolumeNameBuffer
   lpVolumeSerialNumber     : DWORD; // address of volume serial number
   lpMaximumComponentLength : DWORD; // address of system's maximum filename length
   lpFileSystemFlags        : DWORD; // address of file system flags
   lpFileSystemNameBuffer   : PChar; // address of name of file system
   nFileSystemNameSize      : DWORD; // length of lpFileSystemNameBuffer
begin
   try
      GetMem( lpVolumeNameBuffer, MAX_PATH + 1 );
      GetMem( lpFileSystemNameBuffer, MAX_PATH + 1 );

      nVolumeNameSize := MAX_PATH + 1;
      nFileSystemNameSize := MAX_PATH + 1;

      lpRootPathName := PChar( 'C:\' );
      if Windows.GetVolumeInformation( lpRootPathName,
                                       lpVolumeNameBuffer,
                                       nVolumeNameSize,
                                       @lpVolumeSerialNumber,
                                       lpMaximumComponentLength,
                                       lpFileSystemFlags,
                                       lpFileSystemNameBuffer,
                                       nFileSystemNameSize ) then
      begin
        with SysInfoRec do begin
          VolumeName         := lpVolumeNameBuffer;
          VolumeSerial       := IntToHex(HiWord(lpVolumeSerialNumber), 4) + '-' +
                  		IntToHex(LoWord(lpVolumeSerialNumber), 4);
          FileSystemName     := lpFileSystemNameBuffer;
        end;
      end;
   finally
      FreeMem( lpVolumeNameBuffer );
      FreeMem( lpFileSystemNameBuffer );
   end; // try
end; {- GetVolumeInfo }

{===============================================================================
}
procedure GetOSVersionInfo;

  function Plat(Pl: DWORD): string;
  begin
    case Pl of
      VER_PLATFORM_WIN32s:        result := 'Windows 3.1';
      VER_PLATFORM_WIN32_WINDOWS: result := 'Windows 95';
      VER_PLATFORM_WIN32_NT:      result := 'Windows NT';
      else                        result := '???';
    end;
  end;

begin

  with OSVerInfo, SysInfoRec do begin
    dwOSVersionInfoSize := SizeOf(OSVerInfo);
    if GetVersionEx(OSVerInfo) then;
    Version := Format('%d.%d (%d.%s)',[dwMajorVersion, dwMinorVersion,
                                      (dwBuildNumber and $FFFF), szCSDVersion]);
    Plattform := Plat(dwPlatformId);
    PlattID   := dwPlatformId;
  end;
end; {- GetOSVersionInfo }


{===============================================================================
}
procedure GetDriveNames;
var
   D1   : set of 0..25;
   D2   : integer;
begin
   DWORD( D1 ) := Windows.GetLogicalDrives;
   with SysInfoRec do begin
     Drives := '';
     for D2 := 0 to 25 do
        if D2 in D1 then
           Drives := Drives + Chr( D2 + Ord( 'A' )) + ': ';
   end;
end; {- GetDriveNames }


{===============================================================================
}
procedure GetSystemInfo;
var TmpStr: string;
    MProc: string;
    LocalSI: TSystemInfo;
const
   PROCESSOR_INTEL_386     = 386;
   PROCESSOR_INTEL_486     = 486;
   PROCESSOR_INTEL_PENTIUM = 586;
   PROCESSOR_MIPS_R4000    = 4000;
   PROCESSOR_ALPHA_21064   = 21064;
begin
  Windows.GetSystemInfo(LocalSI);

  with LocalSI, SysInfoRec do begin
    case dwProcessorType of
       PROCESSOR_INTEL_386      : ProcessorType := ' 386';
       PROCESSOR_INTEL_486      : ProcessorType := ' 486';
       PROCESSOR_INTEL_PENTIUM  : ProcessorType := ' Pentium';
       PROCESSOR_MIPS_R4000     : ProcessorType := ' MIPS';
       PROCESSOR_ALPHA_21064    : ProcessorType := ' ALPHA';
    end;
  end;
end; {- GetSystemInfo. }


{===============================================================================
}
procedure MemoryInfo;
var
   MemStatus: TMemoryStatus;
begin
  MemStatus.dwLength := SizeOf(MemStatus);
  GlobalMemoryStatus(MemStatus);
  with SysInfoRec do begin
    TotalPhys    := MemStatus.dwTotalPhys DIV 1024;
    AvailPhys    := MemStatus.dwAvailPhys DIV 1024;
    TotalVirtual := MemStatus.dwTotalVirtual DIV 1024;
    AvailVirtual := MemStatus.dwAvailVirtual DIV 1024;
    MemoryLoad   := MemStatus.dwMemoryLoad;
  end;
end; {- MemoryInfo }


{===============================================================================
}
procedure GetRegisterInfo;
const
  FPPKey = '\hardware\DESCRIPTION\System\FloatingPointProcessor';
var
  CurVerKey : PChar;
begin
  with SysInfoRec do begin
    case PlattID of
      VER_PLATFORM_WIN32_WINDOWS :
            CurVerKey := '\SOFTWARE\Microsoft\Windows\CurrentVersion';
      VER_PLATFORM_WIN32_NT      :
            CurVerKey := '\SOFTWARE\Microsoft\Windows NT\CurrentVersion';
      else  CurVerKey := nil;
    end;

    with TRegistry.Create do
    try
        RootKey := HKEY_LOCAL_MACHINE;
        if OpenKey(FPPKey, False) then
           FPU :=  'Yes'
        else FPU :=  'No';
    finally
      Free;
    end; {- try }

    UserName := GetRegStr(CurVerKey,'RegisteredOwner');
    CompanyName := GetRegStr(CurVerKey,'RegisteredOrganization');
    if PlattID = VER_PLATFORM_WIN32_WINDOWS then
       CDSerial :=  GetRegStr(CurVerKey,'ProductID');

  end; {- with }
end; {- GetRegisterInfo }


{===============================================================================
 Get bios info via Registers
}
procedure GetBiosInfo;
begin
  with SysInfoRec do begin
    BiosDate    := GetRegStr('Enum\Root\*PNP0C01\0000','BIOSDate');
    BiosName    := GetRegStr('Enum\Root\*PNP0C01\0000','BIOSName');
    BiosVer     := GetRegStr('Enum\Root\*PNP0C01\0000','BIOSVersion');
    BusType     := GetRegStr('Enum\Root\*PNP0C01\0000','BusType');
    CPU         := GetRegStr('Enum\Root\*PNP0C01\0000','CPU');
    MachineType := GetRegStr('Enum\Root\*PNP0C01\0000','MachineType');
  end;
end; {- GetBiosInfo }


{===============================================================================
}
function GetAllSystemInfo: SystemInfoRecord; stdcall;
begin
  GetOSVersionInfo;
  GetOSVersionInfo;
  GetDriveNames;
  GetSystemInfo;
  GetVolumeInfo;
  GetComputerName;
  MemoryInfo;
  GetRegisterInfo;
  GetBiosInfo;
  result := SysInfoRec;
end; {- GetAllSystemInfo }


{===============================================================================
  This simple functions to test the string result of a function
  Borland suggest the use of shortstring to avoid the use of ShareMem.
}
function DllVersion: shortstring;
begin
  result := 'SysInfo Dll * v. 0.2a  by RicoSoft';
end; {- DllVersion }



end.
